#ifndef SOLVER_H
#define SOLVER_H
#include <vector>
#include "Problem.h"
#include "Control.h"
#include "Random.h"
#include "Solution.h"
#include "Explorer.h"
#include "Runners.h"


  /** The Simple Local Search solver handles a simple local search algorithm
      encapsulated in a runner.
      @ingroup Solvers
  */
class ColoringSolver
  {
  public:
    ColoringSolver(Control *c):p_ctrl(c) {}
    virtual ~ColoringSolver() {};
    virtual void addRunner(ColoringRunner *r) {};
    virtual void Solve(Coloring &current) {}; 
  protected:
    
    Control* p_ctrl;    
 
  };


class ConstructiveHeuristics:public ColoringSolver
{
  public:
  ConstructiveHeuristics(Control* c):ColoringSolver(c) {};
  ConstructiveHeuristics(Control* c,
		ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~ConstructiveHeuristics() {};
  void Solve(Coloring &current);
 protected:
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
};

class RandomRestart:public ColoringSolver
{
  public:
  RandomRestart(Control* c):ColoringSolver(c) {};
  RandomRestart(Control* c,
		ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~RandomRestart() {};
  void Solve(Coloring &current);
 protected:
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
};
 
class Metaheuristic_Fixed:public ColoringSolver
{
 public:
  Metaheuristic_Fixed(Control* c):ColoringSolver(c) {};
  Metaheuristic_Fixed(Control* c,
		ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~Metaheuristic_Fixed() {};
  void SetRunner(ColoringRunner *r) {p_runner = r;};
  void Solve(Coloring &current);
 protected:
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
};

class Metaheuristic_Descend:public ColoringSolver
{
  public:
  Metaheuristic_Descend(Control* c):ColoringSolver(c) {};
  Metaheuristic_Descend(Control* c,
			ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~Metaheuristic_Descend() {};
  void SetRunner(ColoringRunner *r) {p_runner = r;};
  void Solve(Coloring &current);
   protected:
  unsigned long NumberOfIterations() const {return total_iterations;};
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
			     runners. */

};


class ColoringMultiRunnerSolver: public ColoringSolver
{
 public:
  void ClearRunners();
  void addRunner(ColoringRunner *r);
  void setRunner(ColoringRunner *r, unsigned int i);
  void Solve(Coloring &current);
  ColoringMultiRunnerSolver(Control* c):ColoringSolver(c) { runners.clear(); };
  protected:


  //void RunCheck() const throw(EasyLocalException);
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long NumberOfIterations() const {return total_iterations;};;
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
				     runners. */
  unsigned int start_runner;      /**< The index of the runner to
				     start with. */
  vector<ColoringRunner* > runners; /**< The vector of
						 the linked runners. */
};


class Metaheuristic_HEA:public ColoringSolver
{
  public:
  Metaheuristic_HEA(Control* c):ColoringSolver(c) {};
  Metaheuristic_HEA(Control* c,
			ColoringRunner* r, Graph *g, ColoringManager *sm):
    ColoringSolver(c), p_runner(r), gr(g), p_sm(sm) {};
  virtual ~Metaheuristic_HEA() {};
  void SetRunner(ColoringRunner *r) {p_runner = r;};
   virtual void Solve(Coloring &current);
 protected:
  unsigned int SIZE;
  Coloring *crossOver(Coloring *par_a, Coloring *par_b);
  void HEA(Coloring &current);
  double averageDistanceSolutions(Coloring *p_pop[]);
  unsigned long NumberOfIterations() const {return total_iterations;};
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
			     runners. */
 protected:
  Graph *gr;
  ColoringManager *p_sm;
};


class Metaheuristic_HEA_rlf:public Metaheuristic_HEA
{
  public:
  Metaheuristic_HEA_rlf(Control* c):Metaheuristic_HEA(c) {};
  Metaheuristic_HEA_rlf(Control* c,
			ColoringRunner* r, Graph *g, ColoringManager *sm):
    Metaheuristic_HEA(c,r,g,sm) {};
  virtual ~Metaheuristic_HEA_rlf() {};

  void Solve(Coloring &current);
};


class Metaheuristic_HEA_fixed:public Metaheuristic_HEA
{
  public:
  Metaheuristic_HEA_fixed(Control* c):Metaheuristic_HEA(c) {};
  Metaheuristic_HEA_fixed(Control* c,
			ColoringRunner* r, Graph *g, ColoringManager *sm):
    Metaheuristic_HEA(c,r,g,sm) {};
  virtual ~Metaheuristic_HEA_fixed() {};

  void Solve(Coloring &current);
};


class XRLFSolver:public ColoringSolver
{
 public:
  XRLFSolver(Control* c):ColoringSolver(c) {};
  XRLFSolver(Control* c, ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~XRLFSolver() {};
  void SetRunner(ColoringRunner *r) {p_runner = r;};
  void Solve(Coloring &current);
 protected:
  unsigned long NumberOfIterations() const {return total_iterations;};
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
			     runners. */

};


class PenaltyFunction:public ColoringSolver
{
  public:
  PenaltyFunction(Control* c):ColoringSolver(c) {};
  PenaltyFunction(Control* c,
			ColoringRunner* r):ColoringSolver(c), p_runner(r) {};
  virtual ~PenaltyFunction() {};
  //void SetRunner(ColoringRunner *r) {p_runner = r;};
  void Solve(Coloring &current);
 protected:
  unsigned long NumberOfIterations() const {return total_iterations;};
  ColoringRunner* p_runner; /**< A pointer to the managed runner. */
  bool MaxIterationExpired() {return total_iterations>p_ctrl->max_iter;}
  unsigned long total_iterations; /**< The overall number of iterations
				     performed by all the managed 
			     runners. */

};

#endif
