

#ifndef DA_SET_INCLUDED
#define DA_SET_INCLUDED
#include <vector>
#include <algorithm>

using namespace std;

template<typename _Tp>
class da_set {
 
public:
  typedef typename std::vector<_Tp>::size_type size_type;

  typedef typename std::vector<_Tp>::iterator iterator;
  typedef typename std::vector<_Tp>::const_iterator const_iterator;
  typedef typename std::vector<size_type>::const_iterator const_iterator_p;

  typedef _Tp 						value_type;
  typedef value_type* 					pointer;
  typedef const value_type* 				const_pointer;
  typedef value_type& 					reference;
  typedef const value_type& 				const_reference;

  inline size_type size() const {return elems_.size();}
  inline size_type max_size() const {return max_size_;}
  inline size_type capacity() const {return elems_.capacity();}
  inline bool empty() const {return elems_.empty();}

  inline const_iterator_p positions_begin() const {return positions_.begin();}
  inline const_iterator_p positions_end() const {return positions_.end();}

  inline iterator begin() {return elems_.begin();}
  inline iterator end() {return elems_.end();}
  inline const_iterator begin() const {return elems_.begin();}
  inline const_iterator end() const {return elems_.end();}

  inline const_reference operator[](size_type i) const {return elems_[i];}
  inline reference operator[](size_type i) {return elems_[i];}
  inline reference front() const {return elems_.front();}
  inline const_reference front() {return elems_.front();}
  inline reference back() const {return elems_.back();}
  inline const_reference back() {return elems_.back();}
 

  da_set() 
    : max_size_(), elems_(), positions_()
    {}

  da_set(size_type _n) 
    : max_size_(_n), elems_(), positions_()
  {
    elems_.reserve(max_size());
    positions_.resize(max_size_, -1);
  }
  
  da_set(const da_set &a)
    : max_size_(a.max_size()), elems_(), positions_()
  {
    elems_.reserve(max_size());
    positions_.reserve(max_size());
    std::copy(a.begin(), a.end(), std::back_inserter(elems_));
    std::copy(a.positions_begin(), a.positions_end(), 
	      std::back_inserter(positions_));
  }

  ~da_set() {}

  da_set& operator=(const da_set& __x)
    {
      max_size_(__x.max_size());
      elems_.resize(max_size_);
      positions_.resize(max_size_);
      std::copy(__x.begin(), __x.end(), std::back_inserter(elems_));
      std::copy(__x.positions_begin(), __x.positions_end(), 
		std::back_inserter(positions_));
    }

  inline void insert(_Tp v) 
  {
    if (v >= 0 && v < max_size_)
      {
	if (positions_[v] == -1) {
	  size_type index = size();
	  elems_.push_back(v);
	  positions_[v] = index;
	}
      }
    else {
      cerr<<"Trying to insert something that does not fit into da_set"<<endl;
    }
  }

  inline void erase(_Tp v)
  {
    if ( v >= 0 && v < max_size_ )
      {
	if (positions_[v] != -1) {
	  size_type index = positions_[v];
	  positions_[v] = -1;
	  if ( v != back() )
	    {
	      std::swap(elems_[index], elems_.back());
	      positions_[elems_[index]] = index;
	    }
	  elems_.pop_back();
	}
      }
    else
      {
	cerr<<"Trying to remove something that does not fit into da_set"<<endl;
      }
  }

  inline void erase(iterator pos) {erase(*pos);}

  
private:
  size_type max_size_;

  std::vector<_Tp> elems_;
  std::vector<size_t> positions_;
};

#endif



/*
________________________________________________________________________
#include <iostream>
#include "da_set.hpp"

template <typename S>
void report(const S & s)
{
  // Questa non ha nessun bisogno di essere un template, se non
  // per il fatto di svincolarsi dalla typedef all'inizio del main()

  std::cout << "Capacity :" << s.capacity() << std::endl;
  std::cout << "Size     :" << s.size() << std::endl;

  typename S::const_iterator si, se;
  si = s.begin(); se = s.end();
  for (; si != se; ++si) std::cout << *si << " ";
  std::cout << std::endl;

  std::cout << std::endl;
}

int main()
{
  typedef da_set<10> DA_Set;

  DA_Set das;
  DA_Set::iterator si, se;

  das.insert(5);
  das.insert(7);
  das.insert(3);
  das.insert(4);

  report(das);

  DA_Set das2 = das;
  das.erase(7);

  report(das);
  report(das2);

  das2.erase(5);
  report(das2);

  das.insert(1);
  report(das);
}
*/
